﻿using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Shooter {

	/// <summary>
	/// スコア
	/// </summary>
	struct Score : IComparable<Score> {
		public string Name;
		public int Points;

		public int CompareTo(Score other) {
			return other.Points - Points;
		}
	}
	/// <summary>
	/// スコアランキング
	/// </summary>
	class ScoreRanking {
		readonly StringBuilder text = new StringBuilder();
		public readonly List<Score> Scores = new List<Score>(11);

		public ScoreRanking() {
			for (int i = 0; i < 10; i++) {
				Scores.Add(new Score { Name = new string((char)('A' + i), 3), Points = 1000 - 100 * i, });
			}
		}

		public int Join(Score score) {
			for (int i = Scores.Count; --i >= 0; ) {
				if (score.CompareTo(Scores[i]) >= 0) {
					Scores.Insert(i + 1, score);
					Scores.RemoveAt(Scores.Count - 1);
					return i + 1;
				}
			}
			Scores.Insert(0, score);
			Scores.RemoveAt(Scores.Count - 1);
			return 0;
		}

		public void Draw(World world, ClipRenderer renderer) {
			renderer.SpriteBatch.DrawString(world.Font, "SCORE", new Vector2(0x100, 0x20), Color.White);

			Vector2 position = new Vector2(0, 0x50);
			for (int i = 0; i < Scores.Count; i++) {

				text.Clear();
				text.AppendFormat("{0,2}", i + 1);
				position.X = 0x080;
				renderer.SpriteBatch.DrawString(world.SmallFont, text, position, Color.White);

				text.Clear();
				text.AppendFormat("{0:000,000,000}", Scores[i].Points);
				position.X = 0x0D0;
				renderer.SpriteBatch.DrawString(world.SmallFont, text, position, Color.White);

				text.Clear();
				text.AppendFormat("{0}", Scores[i].Name);
				position.X = 0x180;
				renderer.SpriteBatch.DrawString(world.SmallFont, text, position, Color.White);

				position.Y += world.SmallFont.LineSpacing;
			}
		}
	}

	/// <summary>
	/// スコア表示
	/// </summary>
	class ScoreDisplay : Actor {
		readonly StringBuilder text = new StringBuilder();
		public ScoreDisplay(World world)
			: base(world) {
			Kind = Kind.Particle;
		}
		public override void Draw(ClipRenderer renderer) {
			text.Clear();
			text.AppendFormat("SCORE {0:000,000,000}", World.Player.TotalScore);
			renderer.SpriteBatch.DrawString(World.Font, text, position, Color.White);
		}
		Vector2 position = new Vector2(340, 40);
	}

}
